@@ -1,10 +1,11 @@
+from __future__ import absolute_import
+from __future__ import with_statement
+
 import time
 import heapq
 
 from threading import Lock
 
-from kombu.utils import partition
-
 from celery import states
 from celery.datastructures import AttributeDict, LocalCache
 from celery.utils import kwdict
@@ -146,6 +147,9 @@ class Task(Element):
         self.revoked = timestamp
         self.update(states.REVOKED, timestamp, fields)
 
+    def on_unknown_event(self, type, timestamp=None, **fields):
+        self.update(type.upper(), timestamp, fields)
+
     def info(self, fields=None, extra=[]):
         """Information about this task suitable for on-screen display."""
         if fields is None:
@@ -178,20 +182,16 @@ class State(object):
 
     def freeze_while(self, fun, *args, **kwargs):
         clear_after = kwargs.pop("clear_after", False)
-        self._mutex.acquire()
-        try:
-            return fun(*args, **kwargs)
-        finally:
-            if clear_after:
-                self._clear()
-            self._mutex.release()
+        with self._mutex:
+            try:
+                return fun(*args, **kwargs)
+            finally:
+                if clear_after:
+                    self._clear()
 
     def clear_tasks(self, ready=True):
-        self._mutex.acquire()
-        try:
+        with self._mutex:
             return self._clear_tasks(ready)
-        finally:
-            self._mutex.release()
 
     def _clear_tasks(self, ready=True):
         if ready:
@@ -208,11 +208,8 @@ class State(object):
         self.task_count = 0
 
     def clear(self, ready=True):
-        self._mutex.acquire()
-        try:
+        with self._mutex:
             return self._clear(ready)
-        finally:
-            self._mutex.release()
 
     def get_or_create_worker(self, hostname, **kwargs):
         """Get or create worker by hostname."""
@@ -252,19 +249,18 @@ class State(object):
             self.task_count += 1
         if handler:
             handler(**fields)
+        else:
+            task.on_unknown_event(type, **fields)
         task.worker = worker
 
     def event(self, event):
-        self._mutex.acquire()
-        try:
+        with self._mutex:
             return self._dispatch_event(event)
-        finally:
-            self._mutex.release()
 
     def _dispatch_event(self, event):
         self.event_count += 1
         event = kwdict(event)
-        group, _, type = partition(event.pop("type"), "-")
+        group, _, type = event.pop("type").partition("-")
         self.group_handlers[group](type, event)
         if self.event_callback:
             self.event_callback(self, event)
