@@ -1,4 +1,6 @@
 """celery.log"""
+from __future__ import absolute_import
+
 import logging
 import threading
 import sys
@@ -8,7 +10,7 @@ try:
     from multiprocessing import current_process
     from multiprocessing import util as mputil
 except ImportError:
-    current_process = mputil = None
+    current_process = mputil = None  # noqa
 
 from celery import signals
 from celery import current_app
@@ -42,10 +44,11 @@ class ColorFormatter(logging.Formatter):
 
         if self.use_color and color:
             try:
-                record.msg = color(safe_str(record.msg))
-            except Exception:
+                record.msg = str(color(safe_str(record.msg)))
+            except Exception, exc:
                 record.msg = "<Unrepresentable %r: %r>" % (
-                        type(record.msg), traceback.format_stack())
+                        type(record.msg), exc)
+                record.exc_info = sys.exc_info()
 
         # Very ugly, but have to make sure processName is supported
         # by foreign logger instances.
@@ -100,30 +103,23 @@ class Logging(object):
         if colorize is None:
             colorize = self.supports_color(logfile)
 
-        if mputil:
-            try:
-                mputil._logger = None
-            except AttributeError:
-                pass
+        if mputil and hasattr(mputil, "_logger"):
+            mputil._logger = None
         ensure_process_aware_logger()
         receivers = signals.setup_logging.send(sender=None,
-                                               loglevel=loglevel,
-                                               logfile=logfile,
-                                               format=format,
-                                               colorize=colorize)
+                        loglevel=loglevel, logfile=logfile,
+                        format=format, colorize=colorize)
         if not receivers:
             root = logging.getLogger()
 
             if self.app.conf.CELERYD_HIJACK_ROOT_LOGGER:
                 root.handlers = []
 
-            mp = mputil and mputil.get_logger() or None
+            mp = mputil.get_logger() if mputil else None
             for logger in filter(None, (root, mp)):
-                if logger:
-                    self._setup_logger(logger, logfile, format,
-                                       colorize, **kwargs)
-                    logger.setLevel(loglevel)
-                    signals.after_setup_logger.send(sender=None, logger=logger,
+                self._setup_logger(logger, logfile, format, colorize, **kwargs)
+                logger.setLevel(loglevel)
+                signals.after_setup_logger.send(sender=None, logger=logger,
                                         loglevel=loglevel, logfile=logfile,
                                         format=format, colorize=colorize)
         Logging._setup = True
@@ -133,8 +129,7 @@ class Logging(object):
     def _detect_handler(self, logfile=None):
         """Create log handler with either a filename, an open stream
         or :const:`None` (stderr)."""
-        if logfile is None:
-            logfile = sys.__stderr__
+        logfile = sys.__stderr__ if logfile is None else logfile
         if hasattr(logfile, "write"):
             return logging.StreamHandler(logfile)
         return WatchedFileHandler(logfile)
@@ -327,7 +322,7 @@ class SilenceRepeated(object):
         self._iterations = 0
 
     def __call__(self, *msgs):
-        if self._iterations >= self.max_iterations:
+        if not self._iterations or self._iterations >= self.max_iterations:
             for msg in msgs:
                 self.action(msg)
             self._iterations = 0
