@@ -1,31 +1,20 @@
+from __future__ import absolute_import
+
 import os
 import sys
 import errno
-import signal
-try:
-    from setproctitle import setproctitle as _setproctitle
-except ImportError:
-    _setproctitle = None
+import signal as _signal
 
-CAN_DETACH = True
-try:
-    import resource
-except ImportError:
-    CAN_DETACH = False
+from celery.local import try_import
 
-try:
-    import pwd
-except ImportError:
-    pwd = None
-
-try:
-    import grp
-except ImportError:
-    grp = None
+_setproctitle = try_import("setproctitle")
+resource = try_import("resource")
+pwd = try_import("pwd")
+grp = try_import("grp")
 
 DAEMON_UMASK = 0
 DAEMON_WORKDIR = "/"
-DAEMON_REDIRECT_TO = getattr(os, "devnull", "/dev/nulll")
+DAEMON_REDIRECT_TO = getattr(os, "devnull", "/dev/null")
 
 
 class LockFailed(Exception):
@@ -61,12 +50,14 @@ class PIDFile(object):
         except OSError, exc:
             raise LockFailed(str(exc))
         return self
+    __enter__ = acquire
 
     def is_locked(self):
         return os.path.exists(self.path)
 
-    def release(self):
+    def release(self, *args):
         self.remove()
+    __exit__ = release
 
     def read_pid(self):
         try:
@@ -134,76 +125,63 @@ def create_pidlock(pidfile):
 class DaemonContext(object):
     _is_open = False
 
-    def __init__(self, pidfile=None,
-            working_directory=DAEMON_WORKDIR, umask=DAEMON_UMASK, **kwargs):
-        self.working_directory = working_directory
+    def __init__(self, pidfile=None, workdir=DAEMON_WORKDIR,
+            umask=DAEMON_UMASK, **kwargs):
+        self.workdir = workdir
         self.umask = umask
 
-    def detach(self):
+    def open(self):
+        if not self._is_open:
+            self._detach()
+
+            os.chdir(self.workdir)
+            os.umask(self.umask)
+
+            for fd in reversed(range(get_fdmax(default=2048))):
+                try:
+                    os.close(fd)
+                except OSError, exc:
+                    if exc.errno != errno.EBADF:
+                        raise
+
+            os.open(DAEMON_REDIRECT_TO, os.O_RDWR)
+            os.dup2(0, 1)
+            os.dup2(0, 2)
+
+            self._is_open = True
+    __enter__ = open
+
+    def close(self, *args):
+        if self._is_open:
+            self._is_open = False
+    __exit__ = close
+
+    def _detach(self):
         if os.fork() == 0:      # first child
             os.setsid()         # create new session
             if os.fork() > 0:   # second child
                 os._exit(0)
         else:
             os._exit(0)
-
-    def open(self):
-        if self._is_open:
-            return
-
-        self.detach()
-
-        os.chdir(self.working_directory)
-        os.umask(self.umask)
-
-        for fd in reversed(range(get_fdmax(default=2048))):
-            try:
-                os.close(fd)
-            except OSError, exc:
-                if exc.errno != errno.EBADF:
-                    raise
-
-        os.open(DAEMON_REDIRECT_TO, os.O_RDWR)
-        os.dup2(0, 1)
-        os.dup2(0, 2)
-
-        self._is_open = True
-
-    def close(self):
-        if self._is_open:
-            self._is_open = False
+        return self
 
 
-def create_daemon_context(logfile=None, pidfile=None, uid=None, gid=None,
-        **options):
-    if not CAN_DETACH:
-        raise RuntimeError(
-                "This platform does not support detach.")
-
-    # Make sure SIGCLD is using the default handler.
-    reset_signal("SIGCLD")
+def detached(logfile=None, pidfile=None, uid=None, gid=None, umask=0,
+             workdir=None, **opts):
+    if not resource:
+        raise RuntimeError("This platform does not support detach.")
+    workdir = os.getcwd() if workdir is None else workdir
 
+    signals.reset("SIGCLD")  # Make sure SIGCLD is using the default handler.
     set_effective_user(uid=uid, gid=gid)
 
     # Since without stderr any errors will be silently suppressed,
     # we need to know that we have access to the logfile.
-    if logfile:
-        open(logfile, "a").close()
-    if pidfile:
-        # Doesn't actually create the pidfile, but makes sure it's
-        # not stale.
-        create_pidlock(pidfile)
+    logfile and open(logfile, "a").close()
+    # Doesn't actually create the pidfile, but makes sure it's not stale.
+    pidfile and create_pidlock(pidfile)
 
-    defaults = {"umask": lambda: 0,
-                "working_directory": lambda: os.getcwd()}
-
-    for opt_name, opt_default_gen in defaults.items():
-        if opt_name not in options or options[opt_name] is None:
-            options[opt_name] = opt_default_gen()
-
-    context = DaemonContext(**options)
-
-    return context, context.close
+    return DaemonContext(umask=umask, workdir=workdir)
 
 
 def parse_uid(uid):
@@ -245,25 +223,17 @@ def parse_gid(gid):
 def setegid(gid):
     """Set effective group id."""
     gid = parse_gid(gid)
-    if gid != os.getegid():
+    if gid != os.getgid():
         os.setegid(gid)
 
 
 def seteuid(uid):
     """Set effective user id."""
     uid = parse_uid(uid)
-    if uid != os.geteuid():
+    if uid != os.getuid():
         os.seteuid(uid)
 
 
-def setgid(gid):
-    os.setgid(parse_gid(gid))
-
-
-def setuid(uid):
-    os.setuid(parse_uid(uid))
-
-
 def set_effective_user(uid=None, gid=None):
     """Change process privileges to new user/group.
 
@@ -282,65 +252,82 @@ def set_effective_user(uid=None, gid=None):
         # If gid isn't defined, get the primary gid of the uer.
         if not gid and pwd:
             gid = pwd.getpwuid(uid).pw_gid
-        setgid(gid)
-        setuid(uid)
+        setegid(gid)
+        seteuid(uid)
     else:
-        gid and setgid(gid)
+        gid and setegid(gid)
 
 
-def get_signal(signal_name):
-    """Get signal number from signal name."""
-    if not isinstance(signal_name, basestring) or not signal_name.isupper():
-        raise TypeError("signal name must be uppercase string.")
-    if not signal_name.startswith("SIG"):
-        signal_name = "SIG" + signal_name
-    return getattr(signal, signal_name)
+class Signals(object):
+    ignored = _signal.SIG_IGN
+    default = _signal.SIG_DFL
+
+    def supported(self, signal_name):
+        """Returns true value if ``signal_name`` exists on this platform."""
+        try:
+            return self.signum(signal_name)
+        except AttributeError:
+            pass
+
+    def signum(self, signal_name):
+        """Get signal number from signal name."""
+        if isinstance(signal_name, int):
+            return signal_name
+        if not isinstance(signal_name, basestring) \
+                or not signal_name.isupper():
+            raise TypeError("signal name must be uppercase string.")
+        if not signal_name.startswith("SIG"):
+            signal_name = "SIG" + signal_name
+        return getattr(_signal, signal_name)
+
+    def reset(self, *signal_names):
+        """Reset signals to the default signal handler.
+
+        Does nothing if the platform doesn't support signals,
+        or the specified signal in particular.
+
+        """
+        self.update((sig, self.default) for sig in signal_names)
+
+    def ignore(self, *signal_names):
+        """Ignore signal using :const:`SIG_IGN`.
+
+        Does nothing if the platform doesn't support signals,
+        or the specified signal in particular.
+
+        """
+        self.update((sig, self.ignored) for sig in signal_names)
+
+    def __getitem__(self, signal_name):
+        return _signal.getsignal(self.signum(signal_name))
+
+    def __setitem__(self, signal_name, handler):
+        """Install signal handler.
+
+        Does nothing if the current platform doesn't support signals,
+        or the specified signal in particular.
+
+        """
+        try:
+            _signal.signal(self.signum(signal_name), handler)
+        except (AttributeError, ValueError):
+            pass
+
+    def update(self, _d_=None, **sigmap):
+        """Set signal handlers from a mapping."""
+        for signal_name, handler in dict(_d_ or {}, **sigmap).iteritems():
+            self[signal_name] = handler
 
 
-def reset_signal(signal_name):
-    """Reset signal to the default signal handler.
-
-    Does nothing if the platform doesn't support signals,
-    or the specified signal in particular.
-
-    """
-    try:
-        signum = getattr(signal, signal_name)
-        signal.signal(signum, signal.SIG_DFL)
-    except (AttributeError, ValueError):
-        pass
-
-
-def ignore_signal(signal_name):
-    """Ignore signal using :const:`SIG_IGN`.
-
-    Does nothing if the platform doesn't support signals,
-    or the specified signal in particular.
-
-    """
-    try:
-        signum = getattr(signal, signal_name)
-        signal.signal(signum, signal.SIG_IGN)
-    except (AttributeError, ValueError):
-        pass
-
-
-def install_signal_handler(signal_name, handler):
-    """Install a handler.
-
-    Does nothing if the current platform doesn't support signals,
-    or the specified signal in particular.
-
-    """
-    try:
-        signum = getattr(signal, signal_name)
-        signal.signal(signum, handler)
-    except (AttributeError, ValueError):
-        pass
+signals = Signals()
+get_signal = signals.signum                   # compat
+install_signal_handler = signals.__setitem__  # compat
+reset_signal = signals.reset                  # compat
+ignore_signal = signals.ignore                # compat
 
 
 def strargv(argv):
-    arg_start = "manage" in argv[0] and 2 or 1
+    arg_start = 2 if "manage" in argv[0] else 1
     if len(argv) > arg_start:
         return " ".join(argv[arg_start:])
     return ""
@@ -353,9 +340,9 @@ def set_process_title(progname, info=None):
 
     """
     proctitle = "[%s]" % progname
-    proctitle = info and "%s %s" % (proctitle, info) or proctitle
+    proctitle = "%s %s" % (proctitle, info) if info else proctitle
     if _setproctitle:
-        _setproctitle(proctitle)
+        _setproctitle.setproctitle(proctitle)
     return proctitle
 
 
@@ -365,11 +352,12 @@ def set_mp_process_title(progname, info=None, hostname=None):
     Only works if :mod:`setproctitle` is installed.
 
     """
+    if hostname:
+        progname = "%s@%s" % (progname, hostname.split(".")[0])
     try:
         from multiprocessing.process import current_process
     except ImportError:
-        return
-    if hostname:
-        progname = "%s@%s" % (progname, hostname.split(".")[0])
-    return set_process_title("%s:%s" % (progname, current_process().name),
-                             info=info)
+        return set_process_title(progname, info=info)
+    else:
+        return set_process_title("%s:%s" % (progname,
+                                            current_process().name), info=info)
