@@ -1,48 +1,33 @@
+from __future__ import with_statement
+
 import socket
 import sys
 
+from collections import deque
 from datetime import datetime, timedelta
 from Queue import Empty
 
 from kombu.transport.base import Message
 from kombu.connection import BrokerConnection
-from celery.utils.timer2 import Timer
+from mock import Mock, patch
 
 from celery import current_app
 from celery.concurrency.base import BasePool
 from celery.exceptions import SystemTerminate
 from celery.task import task as task_dec
 from celery.task import periodic_task as periodic_task_dec
-from celery.utils import timer2
 from celery.utils import gen_unique_id
 from celery.worker import WorkController
 from celery.worker.buckets import FastQueue
 from celery.worker.job import TaskRequest
 from celery.worker.consumer import Consumer as MainConsumer
-from celery.worker.consumer import QoS, RUN, PREFETCH_COUNT_MAX
+from celery.worker.consumer import QoS, RUN, PREFETCH_COUNT_MAX, CLOSE
 from celery.utils.serialization import pickle
+from celery.utils.timer2 import Timer
 
 from celery.tests.compat import catch_warnings
 from celery.tests.utils import unittest
-from celery.tests.utils import AppCase, execute_context, skip
-
-
-class MockConsumer(object):
-
-    class Channel(object):
-
-        def close(self):
-            pass
-
-    def register_callback(self, cb):
-        pass
-
-    def consume(self):
-        pass
-
-    @property
-    def channel(self):
-        return self.Channel()
+from celery.tests.utils import AppCase
 
 
 class PlaceHolder(object):
@@ -50,8 +35,8 @@ class PlaceHolder(object):
 
 
 class MyKombuConsumer(MainConsumer):
-    broadcast_consumer = MockConsumer()
-    task_consumer = MockConsumer()
+    broadcast_consumer = Mock()
+    task_consumer = Mock()
 
     def __init__(self, *args, **kwargs):
         kwargs.setdefault("pool", BasePool(2))
@@ -101,81 +86,13 @@ def foo_periodic_task():
     return "foo"
 
 
-class MockLogger(object):
-
-    def __init__(self):
-        self.logged = []
-
-    def critical(self, msg, *args, **kwargs):
-        self.logged.append(msg)
-
-    def info(self, msg, *args, **kwargs):
-        self.logged.append(msg)
-
-    def error(self, msg, *args, **kwargs):
-        self.logged.append(msg)
-
-    def debug(self, msg, *args, **kwargs):
-        self.logged.append(msg)
-
-
-class MockBackend(object):
-    _acked = False
-    no_ack_consumers = set()
-
-    def basic_ack(self, delivery_tag):
-        self._acked = True
-
-
-class MockPool(BasePool):
-    _terminated = False
-    _stopped = False
-
-    def __init__(self, *args, **kwargs):
-        self.raise_regular = kwargs.get("raise_regular", False)
-        self.raise_base = kwargs.get("raise_base", False)
-        self.raise_SystemTerminate = kwargs.get("raise_SystemTerminate",
-                                                False)
-
-    def apply_async(self, *args, **kwargs):
-        if self.raise_regular:
-            raise KeyError("some exception")
-        if self.raise_base:
-            raise KeyboardInterrupt("Ctrl+c")
-        if self.raise_SystemTerminate:
-            raise SystemTerminate()
-
-    def start(self):
-        pass
-
-    def stop(self):
-        self._stopped = True
-        return True
-
-    def terminate(self):
-        self._terminated = True
-        self.stop()
-
-
-class MockController(object):
-
-    def __init__(self, w, *args, **kwargs):
-        self._w = w
-        self._stopped = False
-
-    def start(self):
-        self._w["started"] = True
-        self._stopped = False
-
-    def stop(self):
-        self._stopped = True
-
-
-def create_message(backend, **data):
+def create_message(channel, **data):
     data.setdefault("id", gen_unique_id())
-    return Message(backend, body=pickle.dumps(dict(**data)),
+    channel.no_ack_consumers = set()
+    return Message(channel, body=pickle.dumps(dict(**data)),
                    content_type="application/x-python-serialize",
-                   content_encoding="binary")
+                   content_encoding="binary",
+                   delivery_info={"consumer_tag": "mock"})
 
 
 class test_QoS(unittest.TestCase):
@@ -232,15 +149,8 @@ class test_QoS(unittest.TestCase):
         threaded([add, sub])  # n = 2
         self.assertEqual(qos.value, 1000)
 
-    class MockConsumer(object):
-        prefetch_count = 0
-
-        def qos(self, prefetch_size=0, prefetch_count=0, apply_global=False):
-            self.prefetch_count = prefetch_count
-
     def test_exceeds_short(self):
-        consumer = self.MockConsumer()
-        qos = QoS(consumer, PREFETCH_COUNT_MAX - 1,
+        qos = QoS(Mock(), PREFETCH_COUNT_MAX - 1,
                 current_app.log.get_default_logger())
         qos.update()
         self.assertEqual(qos.value, PREFETCH_COUNT_MAX - 1)
@@ -254,17 +164,17 @@ class test_QoS(unittest.TestCase):
         self.assertEqual(qos.value, PREFETCH_COUNT_MAX - 1)
 
     def test_consumer_increment_decrement(self):
-        consumer = self.MockConsumer()
+        consumer = Mock()
         qos = QoS(consumer, 10, current_app.log.get_default_logger())
         qos.update()
         self.assertEqual(qos.value, 10)
-        self.assertEqual(consumer.prefetch_count, 10)
+        self.assertIn({"prefetch_count": 10}, consumer.qos.call_args)
         qos.decrement()
         self.assertEqual(qos.value, 9)
-        self.assertEqual(consumer.prefetch_count, 9)
+        self.assertIn({"prefetch_count": 9}, consumer.qos.call_args)
         qos.decrement_eventually()
         self.assertEqual(qos.value, 8)
-        self.assertEqual(consumer.prefetch_count, 9)
+        self.assertIn({"prefetch_count": 9}, consumer.qos.call_args)
 
         # Does not decrement 0 value
         qos.value = 0
@@ -273,6 +183,22 @@ class test_QoS(unittest.TestCase):
         qos.increment()
         self.assertEqual(qos.value, 0)
 
+    def test_consumer_decrement_eventually(self):
+        consumer = Mock()
+        qos = QoS(consumer, 10, current_app.log.get_default_logger())
+        qos.decrement_eventually()
+        self.assertEqual(qos.value, 9)
+        qos.value = 0
+        qos.decrement_eventually()
+        self.assertEqual(qos.value, 0)
+
+    def test_set(self):
+        consumer = Mock()
+        qos = QoS(consumer, 10, current_app.log.get_default_logger())
+        qos.set(12)
+        self.assertEqual(qos.prev, 12)
+        qos.set(qos.prev)
+
 
 class test_Consumer(unittest.TestCase):
 
@@ -297,6 +223,12 @@ class test_Consumer(unittest.TestCase):
         info = l.info
         self.assertTrue(info["broker"])
 
+    def test_start_when_closed(self):
+        l = MyKombuConsumer(self.ready_queue, self.eta_schedule, self.logger,
+                            send_events=False)
+        l._state = CLOSE
+        l.start()
+
     def test_connection(self):
         l = MyKombuConsumer(self.ready_queue, self.eta_schedule, self.logger,
                            send_events=False)
@@ -306,7 +238,7 @@ class test_Consumer(unittest.TestCase):
 
         l._state = RUN
         l.event_dispatcher = None
-        l.stop_consumers(close=False)
+        l.stop_consumers(close_connection=False)
         self.assertTrue(l.connection)
 
         l._state = RUN
@@ -331,94 +263,79 @@ class test_Consumer(unittest.TestCase):
 
         l = MyKombuConsumer(self.ready_queue, self.eta_schedule, self.logger,
                            send_events=False)
-        eventer = l.event_dispatcher = MockEventDispatcher()
+        eventer = l.event_dispatcher = Mock()
+        eventer.enabled = True
         heart = l.heart = MockHeart()
         l._state = RUN
         l.stop_consumers()
-        self.assertTrue(eventer.closed)
+        self.assertTrue(eventer.close.call_count)
         self.assertTrue(heart.closed)
 
     def test_receive_message_unknown(self):
         l = MyKombuConsumer(self.ready_queue, self.eta_schedule, self.logger,
                            send_events=False)
-        backend = MockBackend()
+        backend = Mock()
         m = create_message(backend, unknown={"baz": "!!!"})
-        l.event_dispatcher = MockEventDispatcher()
+        l.event_dispatcher = Mock()
         l.pidbox_node = MockNode()
 
-        def with_catch_warnings(log):
+        with catch_warnings(record=True) as log:
             l.receive_message(m.decode(), m)
             self.assertTrue(log)
             self.assertIn("unknown message", log[0].message.args[0])
 
-        context = catch_warnings(record=True)
-        execute_context(context, with_catch_warnings)
-
-    def test_receive_message_eta_OverflowError(self):
+    @patch("celery.utils.timer2.to_timestamp")
+    def test_receive_message_eta_OverflowError(self, to_timestamp):
+        to_timestamp.side_effect = OverflowError()
         l = MyKombuConsumer(self.ready_queue, self.eta_schedule, self.logger,
                              send_events=False)
-        backend = MockBackend()
-        called = [False]
-
-        def to_timestamp(d):
-            called[0] = True
-            raise OverflowError()
-
-        m = create_message(backend, task=foo_task.name,
-                                    args=("2, 2"),
-                                    kwargs={},
-                                    eta=datetime.now().isoformat())
-        l.event_dispatcher = MockEventDispatcher()
-        l.pidbox_node = MockNode()
-
-        prev, timer2.to_timestamp = timer2.to_timestamp, to_timestamp
-        try:
-            l.receive_message(m.decode(), m)
-            self.assertTrue(m.acknowledged)
-            self.assertTrue(called[0])
-        finally:
-            timer2.to_timestamp = prev
-
-    def test_receive_message_InvalidTaskError(self):
-        logger = MockLogger()
-        l = MyKombuConsumer(self.ready_queue, self.eta_schedule, logger,
-                           send_events=False)
-        backend = MockBackend()
-        m = create_message(backend, task=foo_task.name,
-            args=(1, 2), kwargs="foobarbaz", id=1)
-        l.event_dispatcher = MockEventDispatcher()
+        m = create_message(Mock(), task=foo_task.name,
+                                   args=("2, 2"),
+                                   kwargs={},
+                                   eta=datetime.now().isoformat())
+        l.event_dispatcher = Mock()
         l.pidbox_node = MockNode()
 
         l.receive_message(m.decode(), m)
-        self.assertIn("Invalid task ignored", logger.logged[0])
+        self.assertTrue(m.acknowledged)
+        self.assertTrue(to_timestamp.call_count)
+
+    def test_receive_message_InvalidTaskError(self):
+        logger = Mock()
+        l = MyKombuConsumer(self.ready_queue, self.eta_schedule, logger,
+                           send_events=False)
+        m = create_message(Mock(), task=foo_task.name,
+                           args=(1, 2), kwargs="foobarbaz", id=1)
+        l.event_dispatcher = Mock()
+        l.pidbox_node = MockNode()
+
+        l.receive_message(m.decode(), m)
+        self.assertIn("Received invalid task message",
+                      logger.error.call_args[0][0])
 
     def test_on_decode_error(self):
-        logger = MockLogger()
+        logger = Mock()
         l = MyKombuConsumer(self.ready_queue, self.eta_schedule, logger,
                            send_events=False)
 
-        class MockMessage(object):
+        class MockMessage(Mock):
             content_type = "application/x-msgpack"
             content_encoding = "binary"
             body = "foobarbaz"
-            acked = False
-
-            def ack(self):
-                self.acked = True
 
         message = MockMessage()
         l.on_decode_error(message, KeyError("foo"))
-        self.assertTrue(message.acked)
-        self.assertIn("Can't decode message body", logger.logged[0])
+        self.assertTrue(message.ack.call_count)
+        self.assertIn("Can't decode message body",
+                      logger.critical.call_args[0][0])
 
     def test_receieve_message(self):
         l = MyKombuConsumer(self.ready_queue, self.eta_schedule, self.logger,
                            send_events=False)
-        backend = MockBackend()
-        m = create_message(backend, task=foo_task.name,
+        m = create_message(Mock(), task=foo_task.name,
                            args=[2, 4, 8], kwargs={})
 
-        l.event_dispatcher = MockEventDispatcher()
+        l.event_dispatcher = Mock()
         l.receive_message(m.decode(), m)
 
         in_bucket = self.ready_queue.get_nowait()
@@ -443,6 +360,47 @@ class test_Consumer(unittest.TestCase):
         l.connection_errors = (KeyError, )
         self.assertRaises(SyntaxError, l.start)
         l.heart.stop()
+        l.priority_timer.stop()
+
+    def test_consume_messages_ignores_socket_timeout(self):
+
+        class Connection(current_app.broker_connection().__class__):
+            obj = None
+
+            def drain_events(self, **kwargs):
+                self.obj.connection = None
+                raise socket.timeout(10)
+
+        l = MyKombuConsumer(self.ready_queue, self.eta_schedule, self.logger,
+                            send_events=False)
+        l.connection = Connection()
+        l.task_consumer = Mock()
+        l.connection.obj = l
+        l.qos = QoS(l.task_consumer, 10, l.logger)
+        l.consume_messages()
+
+    def test_consume_messages_when_socket_error(self):
+
+        class Connection(current_app.broker_connection().__class__):
+            obj = None
+
+            def drain_events(self, **kwargs):
+                self.obj.connection = None
+                raise socket.error("foo")
+
+        l = MyKombuConsumer(self.ready_queue, self.eta_schedule, self.logger,
+                            send_events=False)
+        l._state = RUN
+        c = l.connection = Connection()
+        l.connection.obj = l
+        l.task_consumer = Mock()
+        l.qos = QoS(l.task_consumer, 10, l.logger)
+        with self.assertRaises(socket.error):
+            l.consume_messages()
+
+        l._state = CLOSE
+        l.connection = c
+        l.consume_messages()
 
     def test_consume_messages(self):
 
@@ -452,51 +410,31 @@ class test_Consumer(unittest.TestCase):
             def drain_events(self, **kwargs):
                 self.obj.connection = None
 
-        class Consumer(object):
-            consuming = False
-            prefetch_count = 0
-
-            def consume(self):
-                self.consuming = True
-
-            def qos(self, prefetch_size=0, prefetch_count=0,
-                            apply_global=False):
-                self.prefetch_count = prefetch_count
-
         l = MyKombuConsumer(self.ready_queue, self.eta_schedule, self.logger,
                              send_events=False)
         l.connection = Connection()
         l.connection.obj = l
-        l.task_consumer = Consumer()
+        l.task_consumer = Mock()
         l.qos = QoS(l.task_consumer, 10, l.logger)
 
         l.consume_messages()
         l.consume_messages()
-        self.assertTrue(l.task_consumer.consuming)
-        self.assertEqual(l.task_consumer.prefetch_count, 10)
-
+        self.assertTrue(l.task_consumer.consume.call_count)
+        l.task_consumer.qos.assert_called_with(prefetch_count=10)
         l.qos.decrement()
         l.consume_messages()
-        self.assertEqual(l.task_consumer.prefetch_count, 9)
+        l.task_consumer.qos.assert_called_with(prefetch_count=9)
 
     def test_maybe_conn_error(self):
-
-        def raises(error):
-
-            def fun():
-                raise error
-
-            return fun
-
         l = MyKombuConsumer(self.ready_queue, self.eta_schedule, self.logger,
                              send_events=False)
         l.connection_errors = (KeyError, )
         l.channel_errors = (SyntaxError, )
-        l.maybe_conn_error(raises(AttributeError("foo")))
-        l.maybe_conn_error(raises(KeyError("foo")))
-        l.maybe_conn_error(raises(SyntaxError("foo")))
+        l.maybe_conn_error(Mock(side_effect=AttributeError("foo")))
+        l.maybe_conn_error(Mock(side_effect=KeyError("foo")))
+        l.maybe_conn_error(Mock(side_effect=SyntaxError("foo")))
         self.assertRaises(IndexError, l.maybe_conn_error,
-                raises(IndexError("foo")))
+                Mock(side_effect=IndexError("foo")))
 
     def test_apply_eta_task(self):
         from celery.worker import state
@@ -512,23 +450,16 @@ class test_Consumer(unittest.TestCase):
         self.assertIs(self.ready_queue.get_nowait(), task)
 
     def test_receieve_message_eta_isoformat(self):
-
-        class MockConsumer(object):
-            prefetch_count_incremented = False
-
-            def qos(self, **kwargs):
-                self.prefetch_count_incremented = True
-
         l = MyKombuConsumer(self.ready_queue, self.eta_schedule, self.logger,
                              send_events=False)
-        backend = MockBackend()
-        m = create_message(backend, task=foo_task.name,
+        m = create_message(Mock(), task=foo_task.name,
                            eta=datetime.now().isoformat(),
                            args=[2, 4, 8], kwargs={})
 
-        l.task_consumer = MockConsumer()
+        l.task_consumer = Mock()
         l.qos = QoS(l.task_consumer, l.initial_prefetch_count, l.logger)
-        l.event_dispatcher = MockEventDispatcher()
+        l.event_dispatcher = Mock()
+        l.enabled = False
         l.receive_message(m.decode(), m)
         l.eta_schedule.stop()
 
@@ -538,14 +469,34 @@ class test_Consumer(unittest.TestCase):
             if item.args[0].task_name == foo_task.name:
                 found = True
         self.assertTrue(found)
-        self.assertTrue(l.task_consumer.prefetch_count_incremented)
+        self.assertTrue(l.task_consumer.qos.call_count)
         l.eta_schedule.stop()
 
+    def test_on_control(self):
+        l = MyKombuConsumer(self.ready_queue, self.eta_schedule, self.logger,
+                             send_events=False)
+        l.pidbox_node = Mock()
+        l.reset_pidbox_node = Mock()
+
+        l.on_control("foo", "bar")
+        l.pidbox_node.handle_message.assert_called_with("foo", "bar")
+
+        l.pidbox_node = Mock()
+        l.pidbox_node.handle_message.side_effect = KeyError("foo")
+        l.on_control("foo", "bar")
+        l.pidbox_node.handle_message.assert_called_with("foo", "bar")
+
+        l.pidbox_node = Mock()
+        l.pidbox_node.handle_message.side_effect = ValueError("foo")
+        l.on_control("foo", "bar")
+        l.pidbox_node.handle_message.assert_called_with("foo", "bar")
+        l.reset_pidbox_node.assert_called_with()
+
     def test_revoke(self):
         ready_queue = FastQueue()
         l = MyKombuConsumer(ready_queue, self.eta_schedule, self.logger,
                            send_events=False)
-        backend = MockBackend()
+        backend = Mock()
         id = gen_unique_id()
         t = create_message(backend, task=foo_task.name, args=[2, 4, 8],
                            kwargs={}, id=id)
@@ -558,19 +509,40 @@ class test_Consumer(unittest.TestCase):
     def test_receieve_message_not_registered(self):
         l = MyKombuConsumer(self.ready_queue, self.eta_schedule, self.logger,
                           send_events=False)
-        backend = MockBackend()
+        backend = Mock()
         m = create_message(backend, task="x.X.31x", args=[2, 4, 8], kwargs={})
 
-        l.event_dispatcher = MockEventDispatcher()
+        l.event_dispatcher = Mock()
         self.assertFalse(l.receive_message(m.decode(), m))
         self.assertRaises(Empty, self.ready_queue.get_nowait)
         self.assertTrue(self.eta_schedule.empty())
 
+    def test_receieve_message_ack_raises(self):
+        l = MyKombuConsumer(self.ready_queue, self.eta_schedule, self.logger,
+                          send_events=False)
+        backend = Mock()
+        m = create_message(backend, args=[2, 4, 8], kwargs={})
+
+        l.event_dispatcher = Mock()
+        l.connection_errors = (socket.error, )
+        l.logger = Mock()
+        m.ack = Mock()
+        m.ack.side_effect = socket.error("foo")
+        with catch_warnings(record=True) as log:
+            self.assertFalse(l.receive_message(m.decode(), m))
+            self.assertTrue(log)
+            self.assertIn("unknown message", log[0].message.args[0])
+        self.assertRaises(Empty, self.ready_queue.get_nowait)
+        self.assertTrue(self.eta_schedule.empty())
+        m.ack.assert_called_with()
+        self.assertTrue(l.logger.critical.call_count)
+
     def test_receieve_message_eta(self):
         l = MyKombuConsumer(self.ready_queue, self.eta_schedule, self.logger,
                           send_events=False)
-        l.event_dispatcher = MockEventDispatcher()
-        backend = MockBackend()
+        l.event_dispatcher = Mock()
+        l.event_dispatcher._outbound_buffer = deque()
+        backend = Mock()
         m = create_message(backend, task=foo_task.name,
                            args=[2, 4, 8], kwargs={},
                            eta=(datetime.now() +
@@ -584,7 +556,7 @@ class test_Consumer(unittest.TestCase):
         finally:
             l.app.conf.BROKER_CONNECTION_RETRY = p
         l.stop_consumers()
-        l.event_dispatcher = MockEventDispatcher()
+        l.event_dispatcher = Mock()
         l.receive_message(m.decode(), m)
         l.eta_schedule.stop()
         in_hold = self.eta_schedule.queue[0]
@@ -596,6 +568,59 @@ class test_Consumer(unittest.TestCase):
         self.assertEqual(task.execute(), 2 * 4 * 8)
         self.assertRaises(Empty, self.ready_queue.get_nowait)
 
+    def test_reset_pidbox_node(self):
+        l = MyKombuConsumer(self.ready_queue, self.eta_schedule, self.logger,
+                          send_events=False)
+        l.pidbox_node = Mock()
+        chan = l.pidbox_node.channel = Mock()
+        l.connection = Mock()
+        chan.close.side_effect = socket.error("foo")
+        l.connection_errors = (socket.error, )
+        l.reset_pidbox_node()
+        chan.close.assert_called_with()
+
+    def test_reset_pidbox_node_green(self):
+        l = MyKombuConsumer(self.ready_queue, self.eta_schedule, self.logger,
+                          send_events=False)
+        l.pool = Mock()
+        l.pool.is_green = True
+        l.reset_pidbox_node()
+        l.pool.spawn_n.assert_called_with(l._green_pidbox_node)
+
+    def test__green_pidbox_node(self):
+        l = MyKombuConsumer(self.ready_queue, self.eta_schedule, self.logger,
+                          send_events=False)
+        l.pidbox_node = Mock()
+
+        connections = []
+
+        class Connection(object):
+
+            def __init__(self, obj):
+                connections.append(self)
+                self.obj = obj
+                self.closed = False
+
+            def channel(self):
+                return Mock()
+
+            def drain_events(self):
+                self.obj.connection = None
+
+            def close(self):
+                self.closed = True
+
+        l.connection = Mock()
+        l._open_connection = lambda: Connection(obj=l)
+        l._green_pidbox_node()
+
+        l.pidbox_node.listen.assert_called_with(callback=l.on_control)
+        self.assertTrue(l.broadcast_consumer)
+        l.broadcast_consumer.consume.assert_called_with()
+
+        self.assertIsNone(l.connection)
+        self.assertTrue(connections[0].closed)
+
     def test_start__consume_messages(self):
 
         class _QoS(object):
@@ -607,21 +632,16 @@ class test_Consumer(unittest.TestCase):
 
         class _Consumer(MyKombuConsumer):
             iterations = 0
-            wait_method = None
 
             def reset_connection(self):
                 if self.iterations >= 1:
                     raise KeyError("foo")
 
-        called_back = [False]
-
-        def init_callback(consumer):
-            called_back[0] = True
-
+        init_callback = Mock()
         l = _Consumer(self.ready_queue, self.eta_schedule, self.logger,
                       send_events=False, init_callback=init_callback)
-        l.task_consumer = MockConsumer()
-        l.broadcast_consumer = MockConsumer()
+        l.task_consumer = Mock()
+        l.broadcast_consumer = Mock()
         l.qos = _QoS()
         l.connection = BrokerConnection()
         l.iterations = 0
@@ -635,25 +655,27 @@ class test_Consumer(unittest.TestCase):
 
         l.consume_messages = raises_KeyError
         self.assertRaises(KeyError, l.start)
-        self.assertTrue(called_back[0])
+        self.assertTrue(init_callback.call_count)
         self.assertEqual(l.iterations, 1)
         self.assertEqual(l.qos.prev, l.qos.value)
 
+        init_callback.reset_mock()
         l = _Consumer(self.ready_queue, self.eta_schedule, self.logger,
                       send_events=False, init_callback=init_callback)
         l.qos = _QoS()
-        l.task_consumer = MockConsumer()
-        l.broadcast_consumer = MockConsumer()
+        l.task_consumer = Mock()
+        l.broadcast_consumer = Mock()
         l.connection = BrokerConnection()
-
-        def raises_socket_error(limit=None):
-            l.iterations = 1
-            raise socket.error("foo")
-
-        l.consume_messages = raises_socket_error
+        l.consume_messages = Mock(side_effect=socket.error("foo"))
         self.assertRaises(socket.error, l.start)
-        self.assertTrue(called_back[0])
-        self.assertEqual(l.iterations, 1)
+        self.assertTrue(init_callback.call_count)
+        self.assertTrue(l.consume_messages.call_count)
+
+    def test_reset_connection_with_no_node(self):
+
+        l = MainConsumer(self.ready_queue, self.eta_schedule, self.logger)
+        self.assertEqual(None, l.pool)
+        l.reset_connection()
 
 
 class test_WorkController(AppCase):
@@ -663,58 +685,34 @@ class test_WorkController(AppCase):
 
     def create_worker(self, **kw):
         worker = WorkController(concurrency=1, loglevel=0, **kw)
-        worker.logger = MockLogger()
+        worker.logger = Mock()
         return worker
 
-    def test_process_initializer(self):
+    @patch("celery.platforms.signals")
+    @patch("celery.platforms.set_mp_process_title")
+    def test_process_initializer(self, set_mp_process_title, _signals):
         from celery import Celery
-        from celery import platforms
         from celery import signals
         from celery.app import _tls
         from celery.worker import process_initializer
         from celery.worker import WORKER_SIGRESET, WORKER_SIGIGNORE
 
-        ignored_signals = []
-        reset_signals = []
-        worker_init = [False]
-        default_app = current_app
-        app = Celery(loader="default", set_as_current=False)
-
-        class Loader(object):
-
-            def init_worker(self):
-                worker_init[0] = True
-        app.loader = Loader()
-
         def on_worker_process_init(**kwargs):
             on_worker_process_init.called = True
         on_worker_process_init.called = False
         signals.worker_process_init.connect(on_worker_process_init)
 
-        def set_mp_process_title(title, hostname=None):
-            set_mp_process_title.called = (title, hostname)
-        set_mp_process_title.called = ()
-
-        pignore_signal = platforms.ignore_signal
-        preset_signal = platforms.reset_signal
-        psetproctitle = platforms.set_mp_process_title
-        platforms.ignore_signal = lambda sig: ignored_signals.append(sig)
-        platforms.reset_signal = lambda sig: reset_signals.append(sig)
-        platforms.set_mp_process_title = set_mp_process_title
-        try:
-            process_initializer(app, "awesome.worker.com")
-            self.assertItemsEqual(ignored_signals, WORKER_SIGIGNORE)
-            self.assertItemsEqual(reset_signals, WORKER_SIGRESET)
-            self.assertTrue(worker_init[0])
-            self.assertTrue(on_worker_process_init.called)
-            self.assertIs(_tls.current_app, app)
-            self.assertTupleEqual(set_mp_process_title.called,
-                                  ("celeryd", "awesome.worker.com"))
-        finally:
-            platforms.ignore_signal = pignore_signal
-            platforms.reset_signal = preset_signal
-            platforms.set_mp_process_title = psetproctitle
-            default_app.set_current()
+        app = Celery(loader=Mock(), set_as_current=False)
+        process_initializer(app, "awesome.worker.com")
+        self.assertIn((tuple(WORKER_SIGIGNORE), {}),
+                      _signals.ignore.call_args_list)
+        self.assertIn((tuple(WORKER_SIGRESET), {}),
+                      _signals.reset.call_args_list)
+        self.assertTrue(app.loader.init_worker.call_count)
+        self.assertTrue(on_worker_process_init.called)
+        self.assertIs(_tls.current_app, app)
+        set_mp_process_title.assert_called_with("celeryd",
+                        hostname="awesome.worker.com")
 
     def test_with_rate_limits_disabled(self):
         worker = WorkController(concurrency=1, loglevel=0,
@@ -760,7 +758,7 @@ class test_WorkController(AppCase):
 
     def test_on_timer_error(self):
         worker = WorkController(concurrency=1, loglevel=0)
-        worker.logger = MockLogger()
+        worker.logger = Mock()
 
         try:
             raise KeyError("foo")
@@ -768,32 +766,34 @@ class test_WorkController(AppCase):
             exc_info = sys.exc_info()
 
         worker.on_timer_error(exc_info)
-        logged = worker.logger.logged[0]
+        logged = worker.logger.error.call_args[0][0]
         self.assertIn("KeyError", logged)
 
     def test_on_timer_tick(self):
         worker = WorkController(concurrency=1, loglevel=10)
-        worker.logger = MockLogger()
+        worker.logger = Mock()
         worker.timer_debug = worker.logger.debug
 
         worker.on_timer_tick(30.0)
-        logged = worker.logger.logged[0]
+        logged = worker.logger.debug.call_args[0][0]
         self.assertIn("30.0", logged)
 
     def test_process_task(self):
         worker = self.worker
-        worker.pool = MockPool()
-        backend = MockBackend()
+        worker.pool = Mock()
+        backend = Mock()
         m = create_message(backend, task=foo_task.name, args=[4, 8, 10],
                            kwargs={})
         task = TaskRequest.from_message(m, m.decode())
         worker.process_task(task)
+        self.assertEqual(worker.pool.apply_async.call_count, 1)
         worker.pool.stop()
 
     def test_process_task_raise_base(self):
         worker = self.worker
-        worker.pool = MockPool(raise_base=True)
-        backend = MockBackend()
+        worker.pool = Mock()
+        worker.pool.apply_async.side_effect = KeyboardInterrupt("Ctrl+C")
+        backend = Mock()
         m = create_message(backend, task=foo_task.name, args=[4, 8, 10],
                            kwargs={})
         task = TaskRequest.from_message(m, m.decode())
@@ -804,8 +804,9 @@ class test_WorkController(AppCase):
 
     def test_process_task_raise_SystemTerminate(self):
         worker = self.worker
-        worker.pool = MockPool(raise_SystemTerminate=True)
-        backend = MockBackend()
+        worker.pool = Mock()
+        worker.pool.apply_async.side_effect = SystemTerminate()
+        backend = Mock()
         m = create_message(backend, task=foo_task.name, args=[4, 8, 10],
                            kwargs={})
         task = TaskRequest.from_message(m, m.decode())
@@ -816,8 +817,9 @@ class test_WorkController(AppCase):
 
     def test_process_task_raise_regular(self):
         worker = self.worker
-        worker.pool = MockPool(raise_regular=True)
-        backend = MockBackend()
+        worker.pool = Mock()
+        worker.pool.apply_async.side_effect = KeyError("some exception")
+        backend = Mock()
         m = create_message(backend, task=foo_task.name, args=[4, 8, 10],
                            kwargs={})
         task = TaskRequest.from_message(m, m.decode())
@@ -825,43 +827,26 @@ class test_WorkController(AppCase):
         worker.pool.stop()
 
     def test_start_catches_base_exceptions(self):
-
-        class Component(object):
-            stopped = False
-            terminated = False
-
-            def __init__(self, exc):
-                self.exc = exc
-
-            def start(self):
-                raise self.exc
-
-            def terminate(self):
-                self.terminated = True
-
-            def stop(self):
-                self.stopped = True
-
         worker1 = self.create_worker()
-        worker1.components = [Component(SystemTerminate())]
+        stc = Mock()
+        stc.start.side_effect = SystemTerminate()
+        worker1.components = [stc]
         self.assertRaises(SystemExit, worker1.start)
-        self.assertTrue(worker1.components[0].terminated)
+        self.assertTrue(stc.terminate.call_count)
 
         worker2 = self.create_worker()
-        worker2.components = [Component(SystemExit())]
+        sec = Mock()
+        sec.start.side_effect = SystemExit()
+        sec.terminate = None
+        worker2.components = [sec]
         self.assertRaises(SystemExit, worker2.start)
-        self.assertTrue(worker2.components[0].stopped)
+        self.assertTrue(sec.stop.call_count)
 
     def test_state_db(self):
         from celery.worker import state
         Persistent = state.Persistent
 
-        class MockPersistent(Persistent):
-
-            def _load(self):
-                return {}
-
-        state.Persistent = MockPersistent
+        state.Persistent = Mock()
         try:
             worker = self.create_worker(db="statefilename")
             self.assertTrue(worker._finalize_db)
@@ -869,7 +854,6 @@ class test_WorkController(AppCase):
         finally:
             state.Persistent = Persistent
 
-    @skip("Issue #264")
     def test_disable_rate_limits(self):
         from celery.worker.buckets import FastQueue
         worker = self.create_worker(disable_rate_limits=True)
@@ -879,37 +863,27 @@ class test_WorkController(AppCase):
 
     def test_start__stop(self):
         worker = self.worker
-        w1 = {"started": False}
-        w2 = {"started": False}
-        w3 = {"started": False}
-        w4 = {"started": False}
-        worker.components = [MockController(w1), MockController(w2),
-                             MockController(w3), MockController(w4)]
+        worker.components = [Mock(), Mock(), Mock(), Mock()]
 
         worker.start()
-        for w in (w1, w2, w3, w4):
-            self.assertTrue(w["started"])
-        self.assertTrue(worker._running, len(worker.components))
+        for w in worker.components:
+            self.assertTrue(w.start.call_count)
         worker.stop()
         for component in worker.components:
-            self.assertTrue(component._stopped)
+            self.assertTrue(w.stop.call_count)
 
     def test_start__terminate(self):
         worker = self.worker
-        w1 = {"started": False}
-        w2 = {"started": False}
-        w3 = {"started": False}
-        w4 = {"started": False}
-        worker.components = [MockController(w1), MockController(w2),
-                             MockController(w3), MockController(w4),
-                             MockPool()]
+        worker.components = [Mock(), Mock(), Mock(), Mock(), Mock()]
+        for component in worker.components[:3]:
+            component.terminate = None
 
         worker.start()
-        for w in (w1, w2, w3, w4):
-            self.assertTrue(w["started"])
+        for w in worker.components[:3]:
+            self.assertTrue(w.start.call_count)
         self.assertTrue(worker._running, len(worker.components))
         self.assertEqual(worker._state, RUN)
         worker.terminate()
-        for component in worker.components:
-            self.assertTrue(component._stopped)
-        self.assertTrue(worker.components[4]._terminated)
+        for component in worker.components[:3]:
+            self.assertTrue(component.stop.call_count)
+        self.assertTrue(worker.components[4].terminate.call_count)
